import $fetch from 'isomorphic-fetch';

// A type of promise-like that resolves synchronously and supports only one observer

const _iteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.iterator || (Symbol.iterator = Symbol("Symbol.iterator"))) : "@@iterator";

const _asyncIteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.asyncIterator || (Symbol.asyncIterator = Symbol("Symbol.asyncIterator"))) : "@@asyncIterator";

// Asynchronously call a function and send errors to recovery continuation
function _catch(body, recover) {
	try {
		var result = body();
	} catch(e) {
		return recover(e);
	}
	if (result && result.then) {
		return result.then(void 0, recover);
	}
	return result;
}

function pipeP() {
  for (var _len = arguments.length, fns = new Array(_len), _key = 0; _key < _len; _key++) {
    fns[_key] = arguments[_key];
  }

  return function (value) {
    return new Promise(function (resolve, reject) {
      var run = function run(_ref, value) {
        var f = _ref[0],
            fns = _ref.slice(1);

        try {
          return Promise.resolve(_catch(function () {
            var _temp = function () {
              if (f === undefined) resolve(value);else return Promise.resolve(f(value)).then(function (_f) {
                run(fns, _f);
              });
            }();

            if (_temp && _temp.then) return _temp.then(function () {});
          }, function (e) {
            reject(e);
          }));
        } catch (e) {
          return Promise.reject(e);
        }
      };

      run(fns, value);
    });
  };
}
var tryCatchP = function tryCatchP(trier, catcher) {
  return function (value) {
    try {
      return Promise.resolve(_catch(function () {
        return Promise.resolve(trier(value));
      }, function (err) {
        return Promise.resolve(catcher(err, value));
      }));
    } catch (e) {
      return Promise.reject(e);
    }
  };
};

var newError = function newError(id, message, res) {
  var error = new Error(message);
  error.id = id;
  if (res) error.res = res;
  return error;
};

var errorIds = {
  RetryError: 'RetryError',
  TimeoutError: 'TimeoutError',
  DecodeResponseError: 'DecodeResponseError',
  InvalidStatusCodeError: 'InvalidStatusCodeError'
};
var delay = function delay(time) {
  return new Promise(function (resolve) {
    setTimeout(function () {
      return resolve();
    }, time);
  });
};

var delayedFail = function delayedFail(timeout) {
  return delay(timeout).then(function () {
    throw newError(errorIds.TimeoutError, 'Timeout');
  });
};

var tapAndDelay = function tapAndDelay(f, time) {
  f(time);
  return delay(time);
};

var nothing = function nothing() {
  return undefined;
};

var delays = {
  delay: delay,
  tapAndDelay: tapAndDelay,
  constant: function constant(time, f) {
    if (time === void 0) {
      time = 1000;
    }

    if (f === void 0) {
      f = nothing;
    }

    return function () {
      return tapAndDelay(f, time);
    };
  },
  exponential: function exponential(time, f) {
    if (time === void 0) {
      time = 1000;
    }

    if (f === void 0) {
      f = nothing;
    }

    return function (i) {
      return tapAndDelay(f, i * i * time);
    };
  },
  limited: function limited(max, delay) {
    return function (i) {
      return delay((i - 1) % max + 1);
    };
  },
  linear: function linear(time, f) {
    if (time === void 0) {
      time = 1000;
    }

    if (f === void 0) {
      f = nothing;
    }

    return function (i) {
      return tapAndDelay(f, i * time);
    };
  }
};
var withBaseUrl = function withBaseUrl(baseUrl) {
  return function (req) {
    req.url = baseUrl + req.url;
    return req;
  };
};
var withHeader = function withHeader(header, value) {
  return function (req) {
    if (req.headers && req.headers instanceof Headers) {
      if (req.headers.has(header)) req.headers.append(header, value);else req.headers.set(header, value);
    } else {
      req.headers = req.headers || {};
      req.headers[header.toLowerCase()] = value;
    }
    return req;
  };
};
var withCredentials = function withCredentials(value) {
  return function (req) {
    req.credentials = value;
    return req;
  };
};
var withEncodedBody = function withEncodedBody(encoder) {
  return function (req) {
    if (req.method && req.method.toLowerCase() !== 'get' && req.body) req.body = encoder(req.body);
    return req;
  };
};
var withJSONEncodedBody =
/*#__PURE__*/
withEncodedBody(JSON.stringify);
var fetch1 = function fetch1(fetch) {
  return function (req) {
    return fetch(req.url, req);
  };
};
var retryable = function retryable(fetch) {
  return function (req) {
    return function () {
      try {
        var retryableErrors = new Set(['ETIMEDOUT', 'ECONNRESET', 'EADDRINUSE', 'ECONNREFUSED', 'EPIPE', 'ENOTFOUND', 'ENETUNREACH', 'EAI_AGAIN']);
        var retryableStatuses = new Set([408, 413, 429, 500, 502, 503, 504]);

        var throwError = function throwError(res) {
          throw newError(errorIds.RetryError, 'Retry error', res);
        };

        return Promise.resolve(_catch(function () {
          return Promise.resolve(fetch(req)).then(function (res) {
            if (retryableStatuses.has(res.status)) throwError(res);
            return res;
          });
        }, function (error) {
          if (retryableErrors.has(error.code)) throwError();
          throw error;
        }));
      } catch (e) {
        return Promise.reject(e);
      }
    };
  };
};
/**
 * @deprecated 6.0.0
 */

var withTimeout = function withTimeout(timeout) {
  return function (fetch) {
    return function () {
      return Promise.race([fetch(), delayedFail(timeout)]);
    };
  };
};
var decodeRetryAfterHeaderValue = function decodeRetryAfterHeaderValue(response) {
  var value = response && response.headers.get('retry-after');
  if (typeof value === 'string') return new Date(value).getTime() - Date.now();
  if (typeof value === 'number') return value;
  return undefined;
};
var withRetry = function withRetry(_temp9) {
  var _ref2 = _temp9 === void 0 ? {} : _temp9,
      _ref2$max = _ref2.max,
      max = _ref2$max === void 0 ? 5 : _ref2$max,
      _ref2$delay = _ref2.delay,
      delay = _ref2$delay === void 0 ? delays.linear() : _ref2$delay,
      _ref2$maxTimeout = _ref2.maxTimeout,
      maxTimeout = _ref2$maxTimeout === void 0 ? Number.MAX_VALUE : _ref2$maxTimeout;

  return function (fetch) {
    return new Promise(function (resolve, reject) {
      var run = function run(i) {
        try {
          var _temp10 = function () {
            if (i === max + 1) reject(newError(errorIds.RetryError, 'Retry failed'));else {
              var _temp11 = _catch(function () {
                return Promise.resolve(fetch()).then(function (_fetch) {
                  resolve(_fetch);
                });
              }, function (error) {
                var retryAfter = decodeRetryAfterHeaderValue(error.res);

                var _temp2 = function () {
                  if (retryAfter > maxTimeout) reject(newError(errorIds.RetryError, 'Retry-after is higher than maxTimeout'));else {
                    return Promise.resolve(retryAfter === undefined ? delay(i) : delays.delay(retryAfter)).then(function () {
                      run(i + 1);
                    });
                  }
                }();

                return _temp2 && _temp2.then ? _temp2.then(function () {}) : void 0;
              });

              if (_temp11 && _temp11.then) return _temp11.then(function () {});
            }
          }();

          return Promise.resolve(_temp10 && _temp10.then ? _temp10.then(function () {}) : void 0);
        } catch (e) {
          return Promise.reject(e);
        }
      };

      run(1);
    });
  };
};
var withSafe204 = function withSafe204(text, json) {
  if (text === void 0) {
    text = '';
  }

  if (json === void 0) {
    json = {};
  }

  return function (res) {
    if (res.status === 204) {
      res.text = function () {
        return Promise.resolve(text);
      };

      res.json = function () {
        return Promise.resolve(json);
      };
    }

    return res;
  };
};
var withClone = function withClone(res) {
  res.cloned = res.clone();
  return res;
};
var decodeResponse = function decodeResponse(res) {
  var contentType = res.headers.get('content-type') || '';
  if (contentType.indexOf('application/json') === 0) return decodeJSONResponse(res);
  if (contentType.indexOf('application/x-www-form-urlencoded') === 0) return decodeFormDataResponse(res);
  if (contentType.indexOf('text/') === 0) return decodeTextResponse(res);
  return res;
};
var decodeTextResponse = function decodeTextResponse(res) {
  try {
    ;
    return Promise.resolve(res.text()).then(function (_res$text) {
      res.data = _res$text;
      return res;
    });
  } catch (e) {
    return Promise.reject(e);
  }
};
var decodeJSONResponse = function decodeJSONResponse(res) {
  try {
    return Promise.resolve(_catch(function () {
      ;
      return Promise.resolve(res.json()).then(function (_res$json) {
        res.data = _res$json;
        return res;
      });
    }, function (e) {
      throw newError(errorIds.DecodeResponseError, e.message, res);
    }));
  } catch (e) {
    return Promise.reject(e);
  }
};
var decodeFormDataResponse = function decodeFormDataResponse(res) {
  try {
    return Promise.resolve(_catch(function () {
      ;
      return Promise.resolve(res.formData()).then(function (_res$formData) {
        res.data = _res$formData;
        return res;
      });
    }, function (e) {
      throw newError(errorIds.DecodeResponseError, e.message, res);
    }));
  } catch (e) {
    return Promise.reject(e);
  }
};
var decodeArrayBufferResponse = function decodeArrayBufferResponse(res) {
  try {
    ;
    return Promise.resolve(res.arrayBuffer()).then(function (_res$arrayBuffer) {
      res.data = _res$arrayBuffer;
      return res;
    });
  } catch (e) {
    return Promise.reject(e);
  }
};
var decodeBlobResponse = function decodeBlobResponse(res) {
  try {
    ;
    return Promise.resolve(res.blob()).then(function (_res$blob) {
      res.data = _res$blob;
      return res;
    });
  } catch (e) {
    return Promise.reject(e);
  }
};
var checkStatus = function checkStatus(res) {
  if (res.status < 200 || res.status >= 400) throw newError(errorIds.InvalidStatusCodeError, 'Invalid status code: ' + res.status, res);
  return res;
};

var formatError = function formatError(error, x) {
  if (x === void 0) {
    x = 'Fetch error';
  }

  try {
    var formatted = '';

    var append = function append(line, empty) {
      if (empty === void 0) {
        empty = false;
      }

      var n = empty ? '\n' : '';
      formatted += n + line + '\n' + n;
    };

    var message = error.message,
        id = error.id,
        errors = error.errors,
        res = error.res;
    append(x + (id ? " [" + id + "]: " : ': ') + message);

    var _temp12 = function () {
      if (id === errorIds.RetryError) {
        var awaitingErrors = errors.map(function (error, i) {
          return formatError(error, "retry: [" + i + "]");
        });
        return Promise.resolve(Promise.all(awaitingErrors)).then(function (formattedErrors) {
          append(formattedErrors.join(''), true);
        });
      } else {
        var _temp13 = function () {
          if (id === errorIds.DecodeResponseError || id === errorIds.InvalidStatusCodeError) {
            var _temp14 = function () {
              if (res.data) append(res.data, true);else {
                var _temp15 = function () {
                  if (res.cloned) return Promise.resolve(res.cloned.text()).then(function (_res$cloned$text) {
                    append(_res$cloned$text, true);
                  });
                }();

                if (_temp15 && _temp15.then) return _temp15.then(function () {});
              }
            }();

            if (_temp14 && _temp14.then) return _temp14.then(function () {});
          }
        }();

        if (_temp13 && _temp13.then) return _temp13.then(function () {});
      }
    }();

    return Promise.resolve(_temp12 && _temp12.then ? _temp12.then(function () {
      return formatted;
    }) : formatted);
  } catch (e) {
    return Promise.reject(e);
  }
};

var logError = function logError(log) {
  if (log === void 0) {
    log = console.error.bind(console);
  }

  return function (error) {
    try {
      return Promise.resolve(formatError(error)).then(function (_formatError) {
        log(_formatError);
        throw error;
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };
};
var fetch =
/*#__PURE__*/
fetch1($fetch);
var retryableFetch =
/*#__PURE__*/
retryable(fetch);
var json = function json(options) {
  return pipeP(withHeader('Content-Type', 'application/json'), withHeader('Accept', 'application/json'), withJSONEncodedBody, retryableFetch, withRetry(options), withSafe204(), decodeJSONResponse, checkStatus);
};
var text = function text(options) {
  return pipeP(retryableFetch, withRetry(options), withSafe204(), decodeTextResponse, checkStatus);
};

export { checkStatus, decodeArrayBufferResponse, decodeBlobResponse, decodeFormDataResponse, decodeJSONResponse, decodeResponse, decodeRetryAfterHeaderValue, decodeTextResponse, delay, delays, errorIds, fetch, fetch1, json, logError, pipeP, retryable, retryableFetch, text, tryCatchP, withBaseUrl, withClone, withCredentials, withEncodedBody, withHeader, withJSONEncodedBody, withRetry, withSafe204, withTimeout };
//# sourceMappingURL=composable-fetch.esm.js.map
